@@ -1,8 +1,4 @@
-"""
-This file is used to save all project wide constants such as the path of the
-source folder, the project path, etc.
-"""
-
+"""All project wide constants are saved in this module."""
 # Place all your constants here
 import os
 import pathlib
@@ -15,7 +11,6 @@ PROJECT_PATH = pathlib.Path(os.path.dirname(SRC_PATH))
 # Coordinate reference systems (crs)
 WGS84 = "EPSG:4326"  # WGS84 standard crs (latitude, longitude)
 UTM35N = "EPSG:32635"  # https://epsg.io/32635 - preferred crs for chernobyl region
-UCS2000_TM10 = "EPSG:6384"  # https://epsg.io/6384 - reference system for ukraine
 
 # Coordinates
 CHERNOBYL_COORDS_WGS84 = (
